use std::time::{Duration, Instant};
use serde::{Serialize, Deserialize};
use actix::{Actor, Addr, Context};
use crate::websocket;
use crate::server;

#[derive(Serialize, Deserialize, Clone, PartialEq)]
pub enum GameState {
    Creating,
    Guessing
}

#[derive(Serialize, Deserialize, Clone)]
pub struct CreatingEx {
    pub question: String,
    pub letters: Vec<char>
}

#[derive(Serialize, Deserialize, Clone)]
pub struct Round {
    pub exercises: Vec<CreatingEx>
}

#[derive(Serialize, Clone)]
pub struct Player {
    pub nick: String,
    pub submitted_word: Option<String>,
    pub submitted_guess: Option<Vec<(String, String)>>,
}

#[derive(Serialize, Clone)]
pub struct Game {
    pub players: Vec<Player>,
    pub round_number: i32,

    #[serde(skip_serializing)]
    pub round: Round,

    pub state: GameState,
}


impl Game {
    pub fn new() -> Self {
        Self {
            players: vec![],
            round_number: 0,
            round: Round{ exercises: vec![] },
            state: GameState::Creating
        }
    }

    fn get_player(&mut self, nick: &str) -> Option<&mut Player> {
        self.players.iter_mut().find(|x| x.nick == nick)
    }

    pub fn submit_creation(&mut self, player_nick: &str, word: String) -> bool {
        match self.state {
            GameState::Creating => {
                if let Some(player) = self.get_player(player_nick) {
                    player.submitted_word = Some(word);
                    true
                }
                else {
                    false
                }
            },
            _ => false
        }
    }

    pub fn submit_guess(&mut self, player_nick: &str, guess: Vec<(String, String)>) -> bool {
        match self.state {
            GameState::Guessing => {
                if let Some(player) = self.get_player(player_nick) {
                    player.submitted_guess = Some(guess);
                    true
                }
                else {
                    false
                }
            },
            _ => false
        }
    }

    pub fn all_submitted(&self) -> bool {
        match self.state {
            GameState::Creating => {
                self.players.iter().map(|p| p.submitted_word.is_some()).fold(true, |a, b| a && b)
            },
            GameState::Guessing => {
                self.players.iter().map(|p| p.submitted_guess.is_some()).fold(true, |a, b| a && b)
            }
        }
    }

    pub fn next_state(&mut self) {
        if self.state == GameState::Guessing {
            self.clear_submissions();
        }
        self.state = match self.state {
            GameState::Creating => GameState::Guessing,
            GameState::Guessing => GameState::Creating
        }
    }

    pub fn new_round(&mut self) {
        self.round.exercises = vec![
            CreatingEx{ question: "An Alphabet".to_owned(), letters: vec!['a', 'b', 'c'] },
            CreatingEx{ question: "A Betabet".to_owned(), letters: vec!['b', 'c', 'd'] },
            CreatingEx{ question: "A Gammaabet".to_owned(), letters: vec!['c', 'd', 'e'] },
            CreatingEx{ question: "A Deltaabet".to_owned(), letters: vec!['d', 'e', 'f'] },
        ];
    }

    pub fn clear_submissions(&mut self) {
        for p in &mut self.players {
            p.submitted_word = None;
            p.submitted_guess = None;
        }
    }

    pub fn player_join(&mut self, nick: String) {
        self.players.push(Player{ nick: nick, submitted_word: None, submitted_guess: None });
    }
}

